# HR API Client library for java

This is a client library for the UW Madison campus REST HR API. 
The information on the HR API itself, how to obtain access to,
etc. can be found at https://developer.wisc.edu .

#### Including the library in your project

Maven:

```xml
<dependency>
  <groupId>edu.wisc</groupId>
  <artifactId>hr-api-client</artifactId>
  <version>1.0.0</version>
</dependency>
```

You will likely need to include the campus GITLab repository in your POM file:

```xml
<repositories>
    <repository>
        <id>uw-madison-gitlab-hrapi</id>
        <name>UW Madison GITLab</name>
        <url>https://git.doit.wisc.edu/api/v4/projects/87965/packages/maven</url>
    </repository>
</repositories>
```

#### Using the library in your code

First thing you will need to do is create a HrAPIClient object.
Once you have this, you can make calls to the Person API using 
the public methods.

For example, to get information by netid, you can:

````java
import edu.wisc.hrapi.model.SupervisoryOrganization;
import edu.wisc.hrapiclient.NotFoundException;
import edu.wisc.hrapiclient.HrAPIClient;

class mytest {

  public void findMadisonSuporg() {
    try {
      HrAPIClient myClient = new HrAPIClient("<my app id>", "<my app secret>");
      SupervisoryOrganization madison = myClient.getSupervisoryOrg("SO00000013",false,false,false);
    } catch (NotFoundException notfound) {
      log.info("Madison Campus not found>");
    }
  }
}
````


#### Application Exit Hang Fix

If you find that your application when done, hangs around (jvm doesn't exit), 
this is due to a bug in the OKHttp library that could potentially keep some of
its threads active for a period of time. A Fix was added to version 1.0.3 of
the PersonAPI Client to add a "shutdown" method. Simply call this method before
your application exits to force these threads to end.

#### Authentication

Authentication to use the Person API comes from having an approved 
application on https://developer.wisc.edu.  When you have a registered
application with access to the Person API, you can create a pair of API 
Keys. This pair of key/secret is used when creating the PersonAPIClient
object.

No attempt is done upon creating the object to validate the credential. 
There are two methods provided by the library:
 - isAuthenticated()
 - Authenticate()

You do not need to call either of these when using the other methods. The
library will be doing this on your behalf. It will also track how long the
authentication is good for, and renew as needed for long running applications.

Any method may throw the AuthenticationException when called if the
credentials provided were invalid.

#### HR  Data Model

The data model for the Person API is contained in the 
edu.wisc.hrapi.model.* classes. Currently there is only
one class provided by the HR API, and that is encapsulated
in the SupervisoryOrganization object.

The other object currently provided is the SupervisoryOrgStats object. The current API 
only has a fetch a specified Supervisory Organization by ID call and a "fetch all" call.
To accomodate the "fetch all", this library provides a method to see how
may SupervisoryOrgianizations are currently present, and how many pages (or calls to
the method) will be needed. The statistics are provided by the getSupervisoryOrgStatistics()
method.  To fetch all the data, you would then loop the getSupervisoryOrgPage(page) 
method to get data. There is no filter ability to the fetchAll call.

When calling the getSupervisoryOrg(string,boolean,boolean,boolean) method, if you want more
than just the org ID values for the superior, subordinate, allSubordinate data values
returned, you need to set these values to true.  This is the equivelent of using the
"include" parameter in the HR API (you just don't have to know what the values to go
along with it are).


The two model entries (AccessToken, OAuthError) are used to manage the authentication 
processs and can be ignored.

#### Logging

The client doesn't do any logging above the debug level.
However, you can enable the logging intercepter for the 
OkHttpClient. This will log some basic information to the
INFO level about requests going to the Person API.

To enable this logging, call setEnalbeHttpLogging(true).
When you call this method, it will force a re-authentication
as the client needs to rebuild the OkHttpClient object.

#### Java Doc
The java class documentation for the Person API Client can be
found at https://engr.pages.doit.wisc.edu/hr-api-client/apidocs/index.html

#### Version History

Version 1.0.0

- Initial Release
