/**
 * Board of Regents of the University of Wisconsin System
 * licenses this file to you under the Apache License,
 * Version 2.0 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a
 * copy of the License at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package edu.wisc.hrapiclient;

import okhttp3.OkHttpClient;
import okhttp3.logging.HttpLoggingInterceptor;
import retrofit2.Retrofit;
import retrofit2.converter.gson.GsonConverterFactory;


public class OAuthAPIServiceGenerator {

    private static final String BASE_URL = "https://api.wisc.edu";

    private static final Retrofit.Builder builder = new Retrofit.Builder()
            .baseUrl(BASE_URL)
            .addConverterFactory(GsonConverterFactory.create())
            ;

    private static Retrofit retrofit = builder.build();

    private static final OkHttpClient.Builder httpClientBuilder  = new OkHttpClient.Builder();
    private static OkHttpClient httpClient = null;

    private static final HttpLoggingInterceptor logging = new HttpLoggingInterceptor().setLevel(HttpLoggingInterceptor.Level.BASIC);

    public static <S> S createService(Class<S> serviceClass, boolean enableLogging) {
        if (!httpClientBuilder.interceptors().contains(logging)) {
            if (enableLogging)
                httpClientBuilder.addInterceptor(logging);
             httpClient = httpClientBuilder.build();
            builder.client(httpClient);
            retrofit = builder.build();
        }
        return retrofit.create(serviceClass);
    }

    public static <S> S createService(Class<S> serviceClass, OkHttpClient client) {
        httpClient = client;
        builder.client(client);
        retrofit = builder.build();
        return retrofit.create(serviceClass);
    }
}
